/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <efl_extension.h>
#include "$(appName).h"
#include "view.h"
#include "view_defines.h"
#include "view/view_main.h"

#define FONT_SIZE 40

static struct view_info {
	Evas_Object *win;
	Evas_Object *conform;
	Evas_Object *navi;
	Elm_Object_Item *navi_item;
} s_info = {
	.win = NULL,
	.conform = NULL,
	.navi = NULL,
};

static void _delete_win_request_cb(void *data, Evas_Object *obj, void *event_info);
static void _layout_back_cb(void *data, Evas_Object *obj, void *event_info);
static void _get_app_resource(const char *edj_file_in, char *edj_path_out);

/**
 * @brief Creates essential objects: window, conformant and layout.
 * @param[in] user_data The data required for view module initialization.
 * @return This function returns 'EINA_TRUE' if the view is created successfully,
 * otherwise 'EINA_FALSE' is returned.
 */
Eina_Bool view_create(void *user_data)
{
	Evas_Object *main_layout;

	/* Create the window */
	s_info.win = view_create_win(PACKAGE);
	if (s_info.win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a window.");
		return EINA_FALSE;
	}

	/* Create the conformant */
	s_info.conform = view_create_conformant(s_info.win);
	if (s_info.conform == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a conformant");
		return EINA_FALSE;
	}

	s_info.navi = view_create_naviframe(s_info.conform, "elm.swallow.content");
	if (!s_info.navi)
		return EINA_FALSE;

	s_info.navi_item = elm_naviframe_item_push(s_info.navi, "", NULL, NULL, NULL, NULL);
	if (!s_info.navi_item) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_naviframe_item_push() failed.");
		return EINA_FALSE;
	}

	main_layout = view_main_create(s_info.navi, s_info.navi_item);
	if (!main_layout)
		return EINA_FALSE;

	/* Show the window after main view is set up */
	evas_object_show(s_info.win);

	return EINA_TRUE;
}

/**
 * @brief Creates a basic window named package.
 * @param[in] pkg_name Name of the window.
 * @return This function returns the window object if it was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_win(const char *pkg_name)
{
	Evas_Object *win = NULL;

	/*
	 * Window
	 * Create and initialize elm_win.
	 * elm_win is mandatory to manipulate the window.
	 */
	win = elm_win_util_standard_add(pkg_name, pkg_name);
	elm_win_conformant_set(win, EINA_TRUE);
	elm_win_autodel_set(win, EINA_TRUE);
	evas_object_smart_callback_add(win, "delete,request", _delete_win_request_cb, NULL);

	return win;
}

/**
 * @brief Creates a layout object based on edje file.
 * @param[in] parent The parent object for the layout.
 * @param[in] file_name File path of EDJ file to be used.
 * @param[in] group_name Name of a group from EDJ to be set.
 * @return This function returns the layout object if it was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_layout(Evas_Object *parent, const char *file_name, const char *group_name)
{
	char edj_path[PATH_MAX] = {0, };
	Evas_Object *layout = NULL;

	_get_app_resource(file_name, edj_path);

	layout = elm_layout_add(parent);
	if (!layout) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_layout_add() failed.");
		return NULL;
	}

	if (!elm_layout_file_set(layout, edj_path, group_name)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_layout_file_set() failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	eext_object_event_callback_add(layout, EEXT_CALLBACK_BACK, _layout_back_cb, NULL);

	return layout;
}

/**
 * @brief Creates a conformant object.
 * @param[in] win The object to which you want to set this conformant.
 * Conformant is mandatory for base GUI to have proper size.
 * @return This function returns the conformant object if it was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_conformant(Evas_Object *win)
{
	/*
	 * Conformant
	 * Create and initialize elm_conformant.
	 * elm_conformant is mandatory for base GUI to have proper size
	 * when indicator or virtual keypad is visible.
	 */
	Evas_Object *conform = NULL;

	if (win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "window is NULL.");
		return NULL;
	}

	conform = elm_conformant_add(win);
	evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, conform);

	evas_object_show(conform);

	return conform;
}

/**
 * @brief Creates a naviframe object.
 * @param[in] parent The parent object for naviframe object.
 * @param[in] part_name The name of the part within parent object where the naviframe is to be set.
 * @return This function returns the naviframe object if it was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_naviframe(Evas_Object *parent, const char *part_name)
{
	Evas_Object *navi = elm_naviframe_add(parent);
	if (!navi) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_naviframe_add() failed.");
		return NULL;
	}

	elm_naviframe_prev_btn_auto_pushed_set(navi, EINA_FALSE);
	elm_object_part_content_set(parent, part_name, navi);
	eext_object_event_callback_add(navi, EEXT_CALLBACK_BACK, _layout_back_cb, NULL);
	evas_object_show(navi);

	return navi;
}

/**
 * @brief Creates a button object for the parent object.
 * @param[in] parent The parent object for button object.
 * @param[in] caption The caption to be displayed on the button.
 * @paran[in] part_name The name of the part within a parent object where the button is to be set.
 * @param[in] on_click_cb The callback function's handler to be invoked on button click.
 * @param[in] data The user data passed to the callback function.
 * @return The function returns button object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_button(Evas_Object *parent, const char *caption, const char *part_name, Evas_Smart_Cb on_click_cb, void *data)
{
	Evas_Object *button = elm_button_add(parent);
	if (!button) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_button_add() failed.");
		return NULL;
	}

	evas_object_size_hint_align_set(button, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(button, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_text_set(button, caption);

	if (part_name)
		elm_object_part_content_set(parent, part_name, button);

	evas_object_smart_callback_add(button, "clicked", on_click_cb, data);
	evas_object_show(button);

	return button;
}

/**
 * @brief Creates a label object for the parent object.
 * @param[in] parent The parent object for label object.
 * @param[in] caption The caption to be displayed on the label.
 * @param[in] part_name The name of the part within a parent object where the label is to be set.
 * @return The function returns label object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_label(Evas_Object *parent, const char *caption, const char *part_name)
{
	char buff[256] = {0,};
	Evas_Object *label = elm_label_add(parent);
	if (!label) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_label_add() failed.");
		return NULL;
	}

	if (caption) {
		snprintf(buff, sizeof(buff), "<p font_size=%d>%s</p>", FONT_SIZE, caption);
		elm_object_text_set(label, buff);
	}

	evas_object_size_hint_align_set(label, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(label, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	if (part_name)
		elm_object_part_content_set(parent, part_name, label);

	evas_object_show(label);

	return label;
}

/**
 * @brief Internal function which creates a genlist for parent object.
 * @param[in] parent The parent object for a genlist.
 * @param[in] part_name The name of the part within a parent object where the genlist is to be set.
 * @return This function returns a genlist object if is was successfully created,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_genlist(Evas_Object *parent, const char *part_name)
{
	Evas_Object *genlist = elm_genlist_add(parent);
	if (!genlist) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_genlist_add() failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);
	elm_genlist_select_mode_set(genlist, ELM_OBJECT_SELECT_MODE_ALWAYS);
	elm_genlist_highlight_mode_set(genlist, EINA_FALSE);
	elm_scroller_policy_set(genlist, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_AUTO);

	if (part_name)
		elm_object_part_content_set(parent, part_name, genlist);

	evas_object_show(genlist);

	return genlist;
}

/**
 * @brief Function which creates an item class for genlist object.
 * @param[in] item_style The name of the style to be applied to the item.
 * @param[in] on_text_get_cb The callback function invoked on item's text redraw.
 * This callback function is assigned to the func.text_get member of the genlist item class structure.
 * @param[in] on_content_get_cb The callback function invoked on item's content redraw.
 * This callback function is assigned to the func.content_get member of the genlist item class structure.
 * @return The function returns item class if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Elm_Genlist_Item_Class *create_genlist_item_class(const char *item_style, Elm_Genlist_Item_Text_Get_Cb on_text_get_cb, Elm_Genlist_Item_Content_Get_Cb on_content_get_cb)
{
	Elm_Genlist_Item_Class *itc = elm_genlist_item_class_new();
	if (!itc) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_genlist_item_class_new() failed.");
		return NULL;
	}

	itc->item_style = item_style;
	itc->func.text_get = on_text_get_cb;
	itc->func.content_get = on_content_get_cb;
	itc->func.state_get = NULL;
	itc->func.del = NULL;

	return itc;
}

/**
 * @brief Creates a toolbar object for the parent object.
 * @param[in] parent The parent object for toolbar object.
 * @param[in] part_name The part name where the toolbar is to be set.
 * @return This function returns toolbar object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_toolbar(Evas_Object *parent, const char *part_name)
{
	Evas_Object *toolbar = elm_toolbar_add(parent);
	if (!toolbar) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_toolbar_add() failed.");
		return NULL;
	}

	elm_toolbar_transverse_expanded_set(toolbar, EINA_TRUE);
	elm_toolbar_homogeneous_set(toolbar, EINA_TRUE);
	elm_toolbar_shrink_mode_set(toolbar, ELM_TOOLBAR_SHRINK_EXPAND);
	elm_toolbar_select_mode_set(toolbar, ELM_OBJECT_SELECT_MODE_ALWAYS);
	elm_object_style_set(toolbar, "tabbar");

	if (part_name)
		elm_object_part_content_set(parent, part_name, toolbar);

	evas_object_show(toolbar);

	return toolbar;
}

/**
 * @brief Creates a box object for the parent object.
 * @param[in] parent The parent object for box object.
 * @return This function returns box object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_box(Evas_Object *parent)
{
	Evas_Object *box = elm_box_add(parent);
	if (!box) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_box_add() failed.");
		return NULL;
	}

	elm_box_horizontal_set(box, EINA_TRUE);
	elm_box_padding_set(box, 10, 10);
	elm_box_align_set(box, 0.0, 0.5);
	evas_object_show(box);

	return box;
}

/**
 * @brief Creates a check box object for the parent object.
 * @param[in] parent The parent object for check box object.
 * @param[in] state The initial state of the widget (checked/unchecked).
 * @return This function returns check box object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_check(Evas_Object *parent, Eina_Bool state, Evas_Smart_Cb on_change_cb)
{
	Evas_Object *check = elm_check_add(parent);
	if (!check) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_check_add() failed.");
		return NULL;
	}

	elm_check_state_set(check, state);
	elm_object_style_set(check, "on&off");
	evas_object_propagate_events_set(check, EINA_FALSE);
	evas_object_smart_callback_add(check, "changed", on_change_cb, NULL);

	return check;
}

/**
 * @brief Creates a radio button object for the parent object.
 * @param[in] parent The parent object for radio button object.
 * @param[in] group The grouping object.
 * @param[in] value The value assigned to the radio button.
 * @return This function returns radio button object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_radio(Evas_Object *parent, Evas_Object *group, int value)
{
	Evas_Object *radio = elm_radio_add(parent);
	if (!radio) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_radio_add() failed.");
		return NULL;
	}

	elm_radio_value_set(radio, value);
	elm_radio_state_value_set(radio, value);

	if (group)
		elm_radio_group_add(radio, group);

	return radio;
}

/**
 * @brief Creates an edit field embedded into a layout.
 * @param[in] parent The parent object for popup window.
 * @param[in] guide_text The text displayed when the entry widget is empty.
 * @return The function returns layout object containing the entry widget
 * if it was created successfully, otherwise 'NULL' is returned.
 */
Evas_Object *view_create_edit_field(Evas_Object *parent, const char *guide_text)
{
	Evas_Object *entry;
	Evas_Object *layout = elm_layout_add(parent);
	if (!layout) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_layout_add() failed.");
		return NULL;
	}

	elm_layout_theme_set(layout, "layout", "editfield", "singleline");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, 0);

	entry = elm_entry_add(layout);
	if (!entry) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_entry_add() failed.");
		return NULL;
	}

	elm_entry_single_line_set(entry, EINA_TRUE);
	elm_entry_scrollable_set(entry, EINA_FALSE);
	elm_object_part_text_set(entry, "elm.guide", guide_text);
	elm_object_part_content_set(layout, "elm.swallow.content", entry);
	evas_object_show(layout);

	return layout;
}

/**
 * @brief Creates a popup window with two buttons: "Cancel" and "OK".
 * @param[in] title The popup title.
 * @param[in] text The text message.
 * @param[in] ok_button_cb The callback function attached to the 'click' event of the 'OK' button.
 * @param[in] cancel_button_cb The callback function attached to the 'click' event of the 'Cancel' button.
 * @return The function returns popup window object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_popup(const char *title, const char *text, Evas_Smart_Cb ok_button_cb, Evas_Smart_Cb cancel_button_cb)
{
	char buff[256] = {0,};
	Evas_Object *button;
	Evas_Object *popup = elm_popup_add(s_info.win);
	if (!popup) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_popup_add() failed.");
		return NULL;
	}

	if (title)
		elm_object_part_text_set(popup, "title,text", title);

	if (text) {
		snprintf(buff, sizeof(buff), "<align=center>%s</align>", text);
		elm_object_text_set(popup, buff);
	}

	elm_popup_orient_set(popup, ELM_POPUP_ORIENT_CENTER);
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);

	button = view_create_button(popup, "OK", "button1", ok_button_cb, NULL);
	if (!button) {
		evas_object_del(popup);
		return NULL;
	}
	elm_object_style_set(button, "popup");

	if (cancel_button_cb) {
		button = view_create_button(popup, "Cancel", "button2", cancel_button_cb, NULL);
		if (!button) {
			evas_object_del(popup);
			return NULL;
		}
		elm_object_style_set(button, "popup");
	}

	evas_object_show(popup);

	return popup;
}

/**
 * @brief Creates a ctxpopup window.
 * @param[in] relative The relative object for ctxpopup object.
 * @return The function returns ctxpopup window object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_ctxpopup(Evas_Object *relative)
{
	Evas_Coord x = 0;
	Evas_Coord y = 0;
	Evas_Object *ctxpopup = elm_ctxpopup_add(s_info.win);
	if (!ctxpopup) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_ctxpopup_add() failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(ctxpopup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_geometry_get(relative, &x, &y, NULL, NULL);
	evas_object_move(ctxpopup, x, y);

	return ctxpopup;
}

/**
 * @brief Creates a date-time object.
 * @param[in] parent The parent object for datetime widget.
 * @parem[in] part_name The name of the part where the datetime widget is to be set.
 * @param[in] dt Pointer to the date/time structure to be set to the widget.
 * @param[in] format The display format.
 * @return The function returns datetime object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_datetime(Evas_Object *parent, const char *part_name, const char *layout, struct tm *dt)
{
	Evas_Object *datetime = elm_datetime_add(parent);

	if (!datetime) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_datetime_add() failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(datetime, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(datetime, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_style_set(datetime, layout);
	elm_datetime_value_set(datetime, dt);

	if (part_name)
		elm_object_part_content_set(parent, part_name, datetime);

	evas_object_show(datetime);

	return datetime;
}

/**
 * @brief Destroys window and frees its resources.
 */
void view_destroy(void)
{
	if (s_info.win == NULL)
		return;

	evas_object_del(s_info.win);
}

/**
 * @brief Internal callback function invoked when the main window needs to be destroyed.
 * @param[in] data The user data passed to the evas_object_smart_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _delete_win_request_cb(void *data, Evas_Object *obj, void *event_info)
{
	ui_app_exit();
}

/**
 * @brief Internal callback function invoked on HW Back button press.
 * @param[in] data The user data passed to the eext_object_event_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _layout_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	if (s_info.navi_item == elm_naviframe_top_item_get(s_info.navi))
		elm_win_lower(s_info.win);
	else
		elm_naviframe_item_pop(s_info.navi);
}

/**
 * @brief Internal function which creates fully qualified path to the provided resource file.
 * @param[in] edj_file_in The file name.
 * @param[out] edj_path_out The fully qualified path to the edj_file_in file.
 */
static void _get_app_resource(const char *edj_file_in, char *edj_path_out)
{
	char *res_path = app_get_resource_path();
	if (res_path) {
		snprintf(edj_path_out, PATH_MAX, "%s%s", res_path, edj_file_in);
		free(res_path);
	}
}
