/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "$(appName).h"
#include "view.h"
#include "view_defines.h"
#include "view/view_recurrence.h"

#define MAX_REMINDERS_COUNT 5
#define BUTTON_DATE_WIDTH 230
#define BUTTON_TIME_WIDTH 160
#define BUTTON_HEIGHT 80
#define ENTRY_WIDTH 330
#define ENTRY_HEIGHT 80

typedef struct _date_time {
	struct tm *tm;
	const char *format;
} date_time_s;

typedef struct _itc_data {
	const char *caption;
	int id;
} itc_data_s;

static struct view_info {
	Evas_Object *naviframe;
	Evas_Object *genlist;
	Evas_Object *radio_repeat;
	Evas_Object *radio_until;
	Evas_Object *popup;
	Evas_Smart_Cb save_cb;
	recurrence_repeat_data *rec_data;
} s_info = {
	.naviframe = NULL,
	.genlist = NULL,
	.radio_repeat = NULL,
	.radio_until = NULL,
	.popup = NULL,
	.save_cb = NULL,
	.rec_data = NULL,
};

static Eina_Bool _navi_pop_cb(void *data, Elm_Object_Item *it);
static void _save_button_clicked_cb(void *data, Evas_Object *obj, void *event_info);
static void _append_items(void);

/**
 * @brief Adds recurrence setting view.
 * @param[in] navi Main naviframe object.
 * @param[in] recurrence_data Pointer to the recurrence data.
 * @param[in] func Callback function
 * @return This function returns a layout object if is was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_recurrence_create(Evas_Object *navi, recurrence_repeat_data *recurrence_data, Evas_Smart_Cb func)
{
	Evas_Object *button;
	Evas_Object *layout;
	Elm_Object_Item *item;
	int range_type = -1;
	time_t time_now = time(0);
	struct tm *loc_tm = localtime(&time_now);

	s_info.naviframe = navi;
	s_info.rec_data = recurrence_data;
	s_info.save_cb = func;
	if (s_info.rec_data->id == CALENDAR_RECURRENCE_NONE)
		s_info.rec_data->tm = *loc_tm;

	if ((int)s_info.rec_data->range_type != -1)
		range_type = s_info.rec_data->range_type;

	layout = view_create_layout(s_info.naviframe, EDJ_RECURRENCE, GRP_RECURRENCE);
	if (!layout)
		return NULL;

	elm_layout_theme_set(layout, "layout", "application", "default");

	s_info.genlist = view_create_genlist(layout, PART_RECURRENCE_LIST);
	if (!s_info.genlist)
		return NULL;

	s_info.radio_repeat = view_create_radio(s_info.genlist, NULL, s_info.rec_data->id);
	s_info.radio_until = view_create_radio(s_info.genlist, NULL, range_type);

	_append_items();

	item = elm_naviframe_item_push(s_info.naviframe, "Recurrence", NULL, NULL, layout, NULL);
	elm_naviframe_item_pop_cb_set(item, _navi_pop_cb, NULL);

	button = view_create_button(s_info.naviframe, "Save", NULL, _save_button_clicked_cb, NULL);
	if (!button)
		return NULL;

	elm_object_item_part_content_set(item, "toolbar", button);

	return layout;
}

/**
 * @brief Internal callback function invoked on 'Cancel' button click from popup window.
 * This function is responsible for active popup deletion.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _popup_cancel_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	if (!s_info.popup)
		return;

	evas_object_del(s_info.popup);
	s_info.popup = NULL;
}

/**
 * @brief Internal callback function invoked on top item removal from the naviframe.
 * This function is responsible for active popup deletion.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] it The naviframe item being popped.
 * @return This function returns 'EINA_TRUE' to allow naviframe item popping
 * or 'EINA_FALSE' to cancel this operation.
 */
static Eina_Bool _navi_pop_cb(void *data, Elm_Object_Item *it)
{
	_popup_cancel_click_cb(NULL, NULL, NULL);

	return EINA_TRUE;
}

/**
 * @brief Internal callback function invoked on 'Save' button click.
 * This function is responsible for recurrence information setting.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _save_button_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	time_t t = time(0);
	struct tm *curr_time = localtime(&t);
	calendar_recurrence_frequency_e freq = (calendar_recurrence_frequency_e)elm_radio_value_get(s_info.radio_repeat);
	int range = elm_radio_value_get(s_info.radio_until);

	elm_toolbar_item_selected_set(event_info, EINA_FALSE);

	if (freq != CALENDAR_RECURRENCE_NONE && range == -1) {
		s_info.popup = view_create_popup("Warning", "Please select repetition range.", _popup_cancel_click_cb, NULL);
		return;
	} else if (freq != CALENDAR_RECURRENCE_NONE) {
		switch (range) {
		case CALENDAR_RANGE_UNTIL:
			if (difftime(mktime(&s_info.rec_data->tm), mktime(curr_time)) < 0) {
				s_info.popup = view_create_popup("Warning", "Invalid due time selected.", _popup_cancel_click_cb, NULL);
				return;
			}
			break;
		case CALENDAR_RANGE_COUNT:
			if (s_info.rec_data->count <= 0) {
				s_info.popup = view_create_popup("Warning", "The recurrence counter value must be greater than 0.", _popup_cancel_click_cb, NULL);
				return;
			} else if (s_info.rec_data->count > MAX_REMINDERS_COUNT) {
				const char * popup_text = "The recurrence counter value cannot be greater than %d.";
				int popup_text_length = strlen(popup_text);
				char buffer[popup_text_length + 1];
				snprintf(buffer, popup_text_length + 1, popup_text, MAX_REMINDERS_COUNT);
				s_info.popup = view_create_popup("Warning", buffer, _popup_cancel_click_cb, NULL);
				return;
			}
			break;
		default:
			s_info.popup = view_create_popup("Warning", "Unknown error occurred.", _popup_cancel_click_cb, NULL);
			return;
		}
	}

	s_info.rec_data->id = freq;
	s_info.rec_data->range_type = (calendar_range_type_e)range;

	s_info.save_cb(NULL, obj, event_info);

	elm_naviframe_item_pop(s_info.naviframe);
}

/**
 * @brief Internal callback function invoked on 'OK' button click from popup window.
 * This function is responsible for recurrence until date/time acquisition from setting controls
 * and data structure alteration.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _datetime_popup_ok_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *selected_item = elm_genlist_selected_item_get(s_info.genlist);
	Evas_Object *datetime = elm_object_part_content_get(s_info.popup, "default");
	if (!datetime) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_object_part_content_get() failed.");
		return;
	}

	elm_datetime_value_get(datetime, &s_info.rec_data->tm);

	elm_genlist_item_update(selected_item);
	_popup_cancel_click_cb(data, obj, event_info);
}

/**
 * @brief Internal callback function invoked on recurrence until date/time button click.
 * This function is responsible for popup creation with event recurrence until date/time setting controls.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _datetime_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *datetime;
	const char *layout = (const char *)data;

	s_info.popup = view_create_popup(NULL, NULL, _datetime_popup_ok_click_cb, _popup_cancel_click_cb);
	if (!s_info.popup)
		return;

	datetime = view_create_datetime(s_info.popup, "default", layout, &s_info.rec_data->tm);
	if (!datetime) {
		evas_object_del(s_info.popup);
		s_info.popup = NULL;
		return;
	}
}

/**
 * @brief Internal callback function invoked on recurrence count entry text change.
 * This function is responsible for recurrence counter value storage.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _count_entry_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	const char *counter = elm_object_text_get(obj);

	evas_object_size_hint_min_set(obj, ENTRY_WIDTH, ENTRY_HEIGHT);

	if (!counter || strlen(counter) == 0) {
		s_info.rec_data->count = 0;
		return;
	}

	s_info.rec_data->count = atoi(counter);
}

/**
 * @brief Internal callback function invoked on genlist item display.
 * This function is assigned to the func.text_get handler of genlist item's class structure.
 * The text passed to the _append_item() function is displayed.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static char *_get_text_cb(void *data, Evas_Object *obj, const char *part)
{
	const char *caption;

	if (!data)
		return NULL;

	if (!strcmp(part, "elm.text")) {
		caption = ((itc_data_s *)data)->caption;

		if (caption)
			return strdup(caption);
	}

	return NULL;
}

/**
 * @brief Internal callback function invoked on genlist item display (repeat frequency).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains a radio button widget for recurrence frequency setting.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_repeat_content_cb(void *data, Evas_Object *obj, const char *part)
{
	itc_data_s *item_data = (itc_data_s *)data;

	if (!strcmp(part, "elm.swallow.icon"))
		return view_create_radio(s_info.genlist, s_info.radio_repeat, item_data->id);

	return NULL;
}

/**
 * @brief Internal callback function invoked on genlist item display (repeat until).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains a radio button widget for recurrence repeating scheme setting (due date).
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_repeat_until_content_cb(void *data, Evas_Object *obj, const char *part)
{
	static date_time_s date_setting = {.format = "%d %b %Y"};
	static date_time_s time_setting = {.format = "%R"};
	itc_data_s *item_data = (itc_data_s *)data;
	Evas_Object *box;
	Evas_Object *button;
	char time_str[64] = {0};

	if (!strcmp(part, "elm.swallow.icon")) {
		return view_create_radio(s_info.genlist, s_info.radio_until, item_data->id);
	} else if (!strcmp(part, "elm.swallow.end")) {
		box = view_create_box(obj);
		if (!box)
			return NULL;

		date_setting.tm = &s_info.rec_data->tm;
		time_setting.tm = &s_info.rec_data->tm;

		strftime(time_str, sizeof(time_str), date_setting.format, &s_info.rec_data->tm);
		button = view_create_button(box, time_str, NULL, _datetime_button_click_cb, (void *)&date_setting);
		if (!button)
			return NULL;

		evas_object_size_hint_min_set(button, BUTTON_DATE_WIDTH, BUTTON_HEIGHT);
		elm_box_pack_end(box, button);

		strftime(time_str, sizeof(time_str), time_setting.format, &s_info.rec_data->tm);
		button = view_create_button(box, time_str, NULL, _datetime_button_click_cb, (void *)&time_setting);
		if (!button)
			return NULL;

		evas_object_size_hint_min_set(button, BUTTON_TIME_WIDTH, BUTTON_HEIGHT);
		elm_box_pack_end(box, button);

		return box;
	}

	return NULL;
}

/**
 * @brief Internal callback function invoked on genlist item display (repeat count).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains a radio button widget for recurrence repeating scheme setting (count).
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_repeat_count_content_cb(void *data, Evas_Object *obj, const char *part)
{
	itc_data_s *item_data = (itc_data_s *)data;
	Evas_Object *layout;
	Evas_Object *entry;

	if (!strcmp(part, "elm.swallow.icon")) {
		return view_create_radio(s_info.genlist, s_info.radio_until, item_data->id);
	} else if (!strcmp(part, "elm.swallow.end")) {
		layout = view_create_edit_field(s_info.genlist, "Set value");
		entry = elm_object_part_content_get(layout, "elm.swallow.content");

		if (s_info.rec_data->count)
		{
			char buf[2];
			sprintf(buf, "%d", s_info.rec_data->count);

			elm_object_text_set(entry, buf);
		}
		if (entry) {
			elm_entry_input_panel_layout_set(entry, ELM_INPUT_PANEL_LAYOUT_NUMBERONLY);
			evas_object_size_hint_min_set(entry, ENTRY_WIDTH, ENTRY_HEIGHT);
			evas_object_smart_callback_add(entry, "changed", _count_entry_changed_cb, NULL);
		}

		return layout;
	}

	return NULL;
}

/**
 * @brief Internal function which appends new item to the genlist.
 * @param[in] item_style The name of the style to be applied to the item.
 * @param[in] item_data The information structure to be passed to the item class callback functions.
 * @param[in] on_text_get_cb The callback function invoked on item's text redraw.
 * This callback function is assigned to the func.text_get member of the genlist item class structure.
 * @param[in] on_content_get_cb The callback function invoked on item's content redraw.
 * This callback function is assigned to the func.content_get member of the genlist item class structure.
 * @return The function returns the item object if it was successfully appended to the genlist,
 * otherwise NULL is returned.
 */
static Elm_Object_Item *_append_item(const char *item_style, itc_data_s *item_data, Elm_Genlist_Item_Text_Get_Cb on_text_get_cb, Elm_Genlist_Item_Content_Get_Cb on_content_get_cb)
{
	Elm_Genlist_Item_Class *itc;
	Elm_Object_Item *item;

	itc = create_genlist_item_class(item_style, on_text_get_cb, on_content_get_cb);
	if (!itc)
		return NULL;

	item = elm_genlist_item_append(s_info.genlist, itc, (void *)item_data, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (!item) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_genlist_item_append() failed.");
		return NULL;
	}

	elm_genlist_item_class_free(itc);

	return item;
}

/**
 * @brief Internal function which appends the set of items to the genlist.
 * @return This function returns 'true' if all the items were successfully appended to the genlist,
 * otherwise 'false' is returned.
 */
static void _append_items(void)
{
	static itc_data_s item_data[9] = {{"Repeat", 0},
										{"None", CALENDAR_RECURRENCE_NONE},
										{"Daily", CALENDAR_RECURRENCE_DAILY},
										{"Weekly", CALENDAR_RECURRENCE_WEEKLY},
										{"Monthly", CALENDAR_RECURRENCE_MONTHLY},
										{"Yearly", CALENDAR_RECURRENCE_YEARLY},
										{"Repeat until", 0},
										{"Until", CALENDAR_RANGE_UNTIL},
										{"Count", CALENDAR_RANGE_COUNT},};

	_append_item("group_index", &item_data[0], _get_text_cb, NULL);
	_append_item("default", &item_data[1], _get_text_cb, _get_repeat_content_cb);
	_append_item("default", &item_data[2], _get_text_cb, _get_repeat_content_cb);
	_append_item("default", &item_data[3], _get_text_cb, _get_repeat_content_cb);
	_append_item("default", &item_data[4], _get_text_cb, _get_repeat_content_cb);
	_append_item("default", &item_data[5], _get_text_cb, _get_repeat_content_cb);
	_append_item("group_index", &item_data[6], _get_text_cb, NULL);
	_append_item("default", &item_data[7], _get_text_cb, _get_repeat_until_content_cb);
	_append_item("default", &item_data[8], _get_text_cb, _get_repeat_count_content_cb);
}
